#include "Creature.h"
#include <string.h>
#include "IA/utils.h"

#define MAX_TURN_SPEED 10

Creature::Creature(int id,
        Ogre::SceneNode *node,
        Ogre::Entity *entity, 
        ActorType type, 
        Shape* shape, 
        std::vector<Ogre::String> stateMapper,           
        std::vector<Sound3D*> soundList,
        float speed,
        float strength,
        float toughness,
        Attack::ATTACK_TYPE attack1,
        Attack::ATTACK_TYPE attack2, 
        float heightOverFloor) : GameActor(id,node,type, shape,heightOverFloor)

{
	mMaxSpeed=speed;
        mStrength=strength;
        mToughness=toughness;
	mAttack1=attack1;
	mAttack2=attack2;
//        mHeightOverFloor=heightOverFloor;
        mActualHealth=mMaxHealth=1000;
        mStateAnimationMapper=stateMapper;
        mSoundList=soundList;
	//mAnimatedCharacter->setInput(mInputHelper);
	//mAnimatedCharacter->playAnimation(100, Critter::Enums::NO_ANIMATION, Critter::Enums::StockAnimationID_Idle, Critter::Enums::StockAnimationID_Idle, true);
        // Animaciones
      mEntity=entity;
      _currentState = IDLE;
      _previousState = IDLE;
      
      _currentAnimation=entity->getAnimationState(mStateAnimationMapper.at((int)IDLE));
      _currentAnimation->setLoop(true);
        _currentAnimation->setEnabled(true);
      
    
}

Creature::~Creature(void)
{
}

void Creature::setMaxSpeed(float maxSpeed)
{
    mMaxSpeed=maxSpeed;
}

std::vector<Sound3D*> Creature::getSoundList() {
    return mSoundList;
}

Sound3D* Creature::getSound(State state) {
    return mSoundList.at(state);
}

Creature::State Creature::getState() {
    return _currentState;
}

void Creature::setState(State state, bool loop) {  
    if (_currentState==state) {
        return;
    }
    // Cambiamos estado
    _currentState = state;   
    
    
    // Paramos animación
    _currentAnimation->setLoop(false);
    _currentAnimation->setEnabled(false);
    
    // Activamos nueva animación
    Ogre::String stateString=Ogre::String(mStateAnimationMapper.at((int)IDLE));
    if (!Ogre::StringUtil::match(mStateAnimationMapper.at(state),"",false)) {
        stateString=mStateAnimationMapper.at(state);
    }
    _currentAnimation=mEntity->getAnimationState(stateString);

    
    _currentAnimation->setLoop(loop);
    _currentAnimation->setEnabled(true);
    //_currentAnimation->setTimePosition(0.0f);
}



Ogre::Quaternion Creature::getOrientation() {
	return getNode()->getOrientation();
}

Ogre::Vector3 Creature::getPosition() {	
	return getNode()->getPosition();
}

float Creature::getMaxSpeed() {
	return mMaxSpeed;
}

float Creature::getVelocityX() {
	return mVx;
}

float Creature::getVelocityY() {
	return mVy;
}


void Creature::update(long elapsedTime){    
       //Actualizar el tiempo de la animacion
        if (_currentAnimation)
         {
            if (_currentState==_previousState) {
            
                float auxT=(float)elapsedTime/1000;
                _currentAnimation->addTime(auxT*getMaxSpeed()/100);
            }
            else {
                _currentAnimation->setTimePosition(0.0);
            }
            _previousState = _currentState;
         }
        
        setState(IDLE,true);
        rememberPosition();
}

void Creature::getDamage(float damage) {
    float realDamage=damage;//-mToughness;
    if (realDamage<=0) {
        return;
    }
    mActualHealth-=realDamage;
    if (mActualHealth<0) {
        mActualHealth=0;
    }
}

bool Creature::isAlive() {
    return mActualHealth>0;
}


Attack::ATTACK_TYPE Creature::getAttack1() {
	return mAttack1;
}

Attack::ATTACK_TYPE Creature::getAttack2() {
	return mAttack2;
}

void Creature::setHeight(float y) {
    Ogre::Vector3 pos=getNode()->getPosition();
    getNode()->setPosition(pos.x,getHeightOverFloor()+y,pos.z);
    
}

void Creature::applyVelocity(float vx, float vy) {
    
        
     
        Ogre::Vector3 posInicial=getNode()->getPosition();
            Ogre::Vector3 velLocal=Ogre::Vector3(vx,0,vy);

            //Transladamos el nodo               
           
            Ogre::Vector2 finalVelocity=globalVelocity(Ogre::Vector2(vx,vy),getNode()->getOrientation());
            getNode()->setPosition(Ogre::Vector3(posInicial.x+finalVelocity.x/5,posInicial.y,posInicial.z+finalVelocity.y/5));
            
             //giramos el nodo    
            double absVx=Ogre::Math::Abs(vx);
            double absVy=Ogre::Math::Abs(vy);
            Ogre::Vector2 vec=Ogre::Vector2(absVx,absVy);
            
            double modulo=vec.length();
            
            double angle=90.0-(Ogre::Math::ASin(absVy/modulo).valueDegrees());  
            
            //giramos el nodo
            angle=std::min<Ogre::Real>(angle,MAX_TURN_SPEED);
            if (vx>0)
            {
              getNode()->yaw(Ogre::Degree(angle));
            }
            else if (vx<0)
            {
                getNode()->yaw(Ogre::Degree(-angle));
            }
            else
            {
                
            }

	  mVx=vx;
	  mVy=vy;
          
      
      synchronizeBody();
}
/*
void Creature::applyTorque(Ogre::Degree giro) {
	mInputHelper.reset();
	mInputHelper.turn(giro,giro);	
    mAnimatedCharacter->setInput(mInputHelper);
}
*/