/*
 * This file is part of ElSoldadoYLaElfa.
 *
 * 
 * Miguel Garde Vallés (C) 2012
 * <miguelgardev@gmail.com>
 *
 * 
 * ElSoldadoYLaElfa examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * ElSoldadoYLaElfa examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ElSoldadoYLaElfa examples.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "GameActor.h"
#include "Attack.h"
#include "Ogre.h"
#include "PHYSICS/shape.h"
#include "Sound/sound3D.h"

//! Clase que representa los seres vivos del juego.


/**
 *  @author Miguel Garde Vallés
 *  @date 14-01-2012
 *
 *  Modela los seres vivos dentro del juego, ya sean controlables o no.
 *  
 *  Hereda de la clase principal, GameActor, y tiene una serie de estados asociados a sonidos y animaciones
 *  (quieto, moverse, atacar, morir, etc.).
 * 
 *  Cada criatura tiene unos atributos concretos de velocidad, vida, etc.
 *  pero su comportamiento viene determinado por las vistas a la que estén asociadas.
 */

class Creature : public GameActor
{
    
    
public:
      enum State {
            IDLE,
            RUN,
            ATTACK1,
            ATTACK2,
            DIE,
            NUM_STATES,
           
        };
	Creature(int id,
                Ogre::SceneNode *node,
                Ogre::Entity *entity,
                ActorType type, 
                Shape* shape, 
                std::vector<Ogre::String> stateMapper,                 
                std::vector<Sound3D*> soundList,
                float speed,
                float strength,
                float toughness,
                Attack::ATTACK_TYPE attack1=Attack::NONE, 
                Attack::ATTACK_TYPE attack2=Attack::NONE, 
                float heightOverFloor=0);
	~Creature(void);

	Ogre::Quaternion getOrientation();
	Ogre::Vector3 getPosition();

        std::vector<Sound3D*> getSoundList();
        Sound3D* getSound(State state);
        
	float getMaxSpeed();
        
        void setMaxSpeed(float maxSpeed);
	float getVelocityX();
	float getVelocityY();
	float getMass();
	Attack::ATTACK_TYPE getAttack1();
	Attack::ATTACK_TYPE getAttack2();

	void update(long elapsedTime);
        
        /*void Creature::stateDie(Ogre::Real deltaT);
        void Creature::stateAttack(Ogre::Real deltaT);
        void Creature::stateIdle(Ogre::Real deltaT);
        void Creature::stateRun(Ogre::Real deltaT);*/
        
        State getState();
        void setState(State state, bool loop);
        
        void setHeight(float y);
	void applyVelocity(float vx, float vy);
	//void applyTorque(Ogre::Degree giro);
        void getDamage(float damage);
        bool isAlive();

	//son privado, lo pongo aqui para pruebas de IA
	//Critter::AnimatedCharacter* mAnimatedCharacter;
	//Critter::CharacterInputHelper mInputHelper;
        
        
        float mMaxHealth;
        float mActualHealth;

private:
        
	float mVx;
	float mVy;
	float mMaxSpeed;
        float mStrength;
        float mToughness;
	Attack::ATTACK_TYPE mAttack1;
	Attack::ATTACK_TYPE mAttack2;
      
        Ogre::Entity* mEntity;
        
        Ogre::AnimationState* _currentAnimation;
        Ogre::AnimationState* _idleAnimation;
        Ogre::AnimationState* _runAnimation;
        
        State _currentState;
        State _previousState;
        
        std::vector<Ogre::String> mStateAnimationMapper;
        std::vector<Sound3D*> mSoundList;

	//Critter::AnimatedCharacter* mAnimatedCharacter;
	//Critter::CharacterInputHelper mInputHelper;
};
