/*
 * This file is part of ElSoldadoYLaElfa.
 *
 * 
 * Miguel Garde Vallés (C) 2012
 * <miguelgardev@gmail.com>
 *
 * 
 * ElSoldadoYLaElfa examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * ElSoldadoYLaElfa examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ElSoldadoYLaElfa examples.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <Ogre.h>
#include "Elemento.h"
using namespace std;

//! Clase que representa una pantalla del juego.


/**
 *  @author Miguel Garde Vallés
 *  @date 14-01-2012
 *
 *  Representa una pantalla del juego, con su configuración completa,
 *  para poder usarla en el editor de escenarios.
 *  
 *  La clase contiene una lista de los elementos del juego,
 *  así como la configuración de la luz y niebla usadas.
 */

class Escenario
{
public:
	Escenario(char*  nombre, char*  file="");
	~Escenario(void);

	//A�ade o quita un objeto que haya sido colocado en el escenario
	void addElement(Elemento *e);
	void removeElement(Elemento *e);

	//establece la ubicacion en la que aparecera el jugador
	void setInitialPosition(Ogre::Vector3 pos);

	//establece el terreno (suelo) y cielo del escenario a partir de los nombres de las imagenes
	void setTerreno(char* );
	void setCielo(char* );

	//cambia la ambientacion visual
	void setNiebla(Ogre::FogMode tipo,Ogre::ColourValue color,float densidad,float comienzo, float fin);
	void setLuz(Ogre::Light* luz, Ogre::ColourValue ambient);

	bool saveEscenario();
	bool loadEscenario();

	char* mNombre;
	char mCfgFile[100];

	Ogre::Light *mLuz;
private:
	Ogre::ColourValue mAmbientLight;

	Ogre::FogMode mTipoNiebla;
	Ogre::ColourValue mColorNiebla;
	float mDensidadNiebla;
	float mComienzoNiebla;
	float mFinNiebla;

	std::vector<Elemento*> mListaElementos;

	Ogre::Vector3 mPosInicial;
};
