/*
 * This file is part of ElSoldadoYLaElfa.
 *
 * 
 * Miguel Garde Vallés (C) 2012
 * <miguelgardev@gmail.com>
 *
 * 
 * ElSoldadoYLaElfa examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * ElSoldadoYLaElfa examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ElSoldadoYLaElfa examples.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "GardeGames/GardeGames.h"

#include "Creature.h"
#include "Sound/sound3D.h"

//! Clase encargada de representar la emisión de un sonido.


/**
 *  @author Miguel Garde Vallés
 *  @date 19-01-2012
 *
 *  Evento disparado por las vistas de las criaturas en un momento dado.
 *  Se define la fuente del sonido y el tipo,
 *  de modo que se sabe la posición desde la que se emitió
 *  (se puede calcular volumen según distancia y activar el estéreo según el ángulo),
 *  se sabe qué sonido se debe reproducir, y las criaturas que lo deseen, 
 *  pueden interpretar el sonido según el tipo y actuar en consecuencia.
 *  
 */

class Evt_Sonido : public GardeGames::Event 
{
public:
    static char const * const gkName;
    
    enum SOUND_TYPE {
        None,
        Walk,
        Attack,
        Die,
        Howl
    };
    
    Evt_Sonido(GameActor *source, SOUND_TYPE type, Sound3D* sound=0, bool looping=false);
    
    ~Evt_Sonido();
    
    GameActor* mSource;
    SOUND_TYPE mType;
    Sound3D* mSound;
    bool mLooping;
    
private:

};


