#include "GameActor.h"
#include "PHYSICS/collisionManager.h"

GameActor::GameActor(int id,Ogre::SceneNode* node, ActorType type, Shape* shape, float height) 
{
	mId=id;
	mNode=node;
	mType=type;
        
        mOldPosition=mNode->getPosition();
        
        mShapes.push_back(shape);
        mBody=new Body(this,mShapes,Ogre::Vector3::ZERO,Ogre::Vector3::UNIT_SCALE,Ogre::Quaternion::ZERO,type);
        mHeightOverFloor=height;
        synchronizeBody();
}
GameActor::~GameActor(void)
{
}

int GameActor::getId() 
{
	return mId;
}

GameActor::ActorType GameActor::getType() 
{
	return mType;
}

Ogre::SceneNode* GameActor::getNode() 
{
	return mNode;
}

Ogre::Vector3 GameActor::getPosition() {
	return mNode->getPosition();
}

float GameActor::getHeightOverFloor() {
    return mHeightOverFloor;
}

void GameActor::setBody(Body* body) {
    if (mBody==body) {
        return;
    }
    if (mBody) {
        CollisionManager::getSingleton().removeBody(mBody);
        delete mBody;
    }

    mBody = body;

    // Sustituimos las Shapes
    std::vector<Shape*>::iterator i;
    for (i = mShapes.begin(); i != mShapes.end(); ++i)
        if (*i)
           delete *i;
    
    mShapes.clear();

    if (mBody)
        mShapes = mBody->getShapes();

    // Sincronizamos
    synchronizeBody();

}

Body* GameActor::getBody() {
    return mBody;
}

void GameActor::restorePosition(){
    mNode->setPosition(mOldPosition);
    synchronizeBody();
}

void GameActor::rememberPosition() {
    mOldPosition=mNode->getPosition();
}

void GameActor::synchronizeBody(){
    if (mBody && mNode) {
        //printf("NODO: (%f,%f,%f)\n",mNode->getPosition().x,mNode->getPosition().y,mNode->getPosition().z);
        //Ogre::Vector3 vNodo1=mNode->getPosition();
        //Ogre::Vector3 vNodo2=Ogre::Vector3((float)mNode->getPosition().x,(float)mNode->getPosition().y,(float)mNode->getPosition().z);
        Ogre::Vector3 aux=mNode->getPosition();
        aux.y-=getHeightOverFloor();
        mBody->setPosition(aux);
        //Ogre::Vector3 vBody=mBody->getPosition();
        //printf("BODY: (%f,%f,%f)\n",mBody->getPosition().x,mBody->getPosition().y,mBody->getPosition().z);
        //mBody->setScale(mNode->getScale());
        //mBody->setOrientation(mNode->getOrientation());
    }
}