/*
 * This file is part of ElSoldadoYLaElfa.
 *
 * 
 * Miguel Garde Vallés (C) 2012
 * <miguelgardev@gmail.com>
 *
 * 
 * ElSoldadoYLaElfa examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * ElSoldadoYLaElfa examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ElSoldadoYLaElfa examples.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "EventType.h"


//! Clase abstracta que representa un acontecimiento del juego.


/**
 *  @author Miguel Garde Vallés
 *  @date 27-03-2012
 *
 *  Representa un suceso ocurrido en el juego,
 *  habrá clases que quieran saber que el suceso ha tenido lugar,
 *  mediante esta clase tendrán constancia de ello y utilizarán el evento para procesar la información.
 *
 *  Ejemplo:
 *
 *  \code
 *  MiEvento.h:
 *  class MiEvento : Event 
 *  {      
 *      public:
 *              MiEvento(TypeX atributo);
 *              TypeX atributo;
 *  }
 * 
 *  MiEvento.cpp:
 *  MiEvento::MiEvento(TypeX atributo)
 *    : Event("MiEvento")
 *  {
 *      this.atributo = atributo;
 *  }
 *  \endcode
 */

namespace GardeGames {
	class Event
	{
	public:

		explicit Event( char const * const inEventTypeName,
						   float inTime = 0.f )
			: m_type( inEventTypeName ),
			  m_time( inTime )
		{}

		Event( Event const & o )
			: m_type( o.m_type ),
			m_time( o.m_time )
		{}

		virtual ~Event()
		{}

		EventType & getType()
		{
			return m_type;
		}
		
		float getTime()
		{
			return m_time;
		}	
		
	private:
		
		EventType	m_type;     // the identified type of the event
		float       m_time;     // the time at which the event fired

	};
}
