/*
 * This file is part of ElSoldadoYLaElfa.
 *
 * 
 * Miguel Garde Vallés (C) 2012
 * <miguelgardev@gmail.com>
 *
 * 
 * ElSoldadoYLaElfa examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * ElSoldadoYLaElfa examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ElSoldadoYLaElfa examples.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once


//! Clase que representa los inputs de las acciones del juego.

/**
 *  @author Miguel Garde Vallés
 *  @date 27-03-2012
 *
 *  Traduce una o más entradas de teclado (enter, 'w', etc.) 
 *  o ratón (click izquierdo, mover hacia arriba, etc.)
 *  a una acción concreta del juego (saltar, disparar, etc.).
 * 
 *  Ejemplo:
 *
 *  \code
 *  //inicializamos la GameAction diciendo si se va a detectar la primera pulsación o mantener pulsada
 *  GameAction *gameAction = new GameAction("nombre", GameAction::NORMAL);
 * 
 *  //la asociamos a una o varias teclas
 *  inputManager->mapToKey(gameAction,OIS::KC_W);
 *  inputManager->mapToKey(gameAction,OIS::KC_UP);
 * 
 *  //comprobamos si está pulsada cuando lo consideremos
 *  if (gameAction->isPressed())
 *  {
 *      //codigo a ejecutar cuando se esté pulsando la tecla 'w' o la flecha arriba
 *  }
 *  
 *  \endcode
 */

namespace GardeGames
{
    class GameAction
    {
		public:	
			enum Comportamiento {
				/**
					Normal behavior. The isPressed() method returns true
					as long as the key is held down.
				*/
				NORMAL = 0,

				/**
					Initial press behavior. The isPressed() method returns
					true only after the key is first pressed, and not again
					until the key is released and pressed again.
				*/
				DETECT_INITAL_PRESS_ONLY = 1
			};

			__declspec(dllexport) GameAction(char* name, int behavior);
			__declspec(dllexport) GameAction(char* name);
			__declspec(dllexport) GameAction();
			__declspec(dllexport) ~GameAction();
			__declspec(dllexport) char* getName();
			__declspec(dllexport) void reset();
			__declspec(dllexport) void tap();
			__declspec(dllexport) void press();
			__declspec(dllexport) void press(int amount);
			__declspec(dllexport) void release();
			__declspec(dllexport) bool isPressed();
			__declspec(dllexport) int getAmount();


		private:
			enum Estados {
				STATE_RELEASED = 0,
				STATE_PRESSED = 1,
				STATE_WAITING_FOR_RELEASE = 2
			};

			char* name;
			int behavior;
			int amount;
			int state;

	};
}
