/*
 * This file is part of ElSoldadoYLaElfa.
 *
 * 
 * Miguel Garde Vallés (C) 2012
 * <miguelgardev@gmail.com>
 *
 * 
 * ElSoldadoYLaElfa examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * ElSoldadoYLaElfa examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ElSoldadoYLaElfa examples.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <OIS.h>
#include <Ogre.h>
#include "InputManager.h"
#include "GameState.h"
#include "GameStateManager.h"
#include "EventManager.h"


//! Clase que inicia el sistema de juego.

/**
 *  @author Miguel Garde Vallés
 *  @date 27-03-2012
 *
 *  Arranca el juego (inicializa variables de Ogre3D, configura la ventana, el teclado, etc.).
 * 
 *  NOTA: no confundir con GameManager. Ésta clase sí contiene el sistema gestor de eventos.
 *  
 *  Ejemplo:
 *
 *  \code
 *  main()
 *  {
 *      GameManagerEventManaged *gameManager = new GameManagerEventManaged();
 *	try
 *      {
 *              std::vector<GameState*> estados;
 *		estados.push_back(new EstadoJugando());
 *		//estados.push_back(new EstadoMenu());
 *		//estados.push_back(new EstadoFinal());
 *		gameManager->startGame(estados,"EstadoJugando");
 *	}
 *	catch (...)
 *      {
 *      }
 *  }
 *  
 *  \endcode
 */


namespace GardeGames {
	class GameManagerEventManaged : Ogre::WindowEventListener {
	public:
		__declspec(dllexport)	GameManagerEventManaged( void );
		__declspec(dllexport)	~GameManagerEventManaged( void );
		
		__declspec(dllexport)	void startGame(std::vector<GameState*> estados, std::string estadoInicial);
		__declspec(dllexport)	void requestShutdown( void );
		
	private:
		/*__declspec(dllexport)*/	GameManagerEventManaged( const GameManagerEventManaged& ) { }
		/*__declspec(dllexport)*/	GameManagerEventManaged & operator = ( const GameManagerEventManaged& );

		bool configureGame( void );
		void setupResources( Ogre::String mResourcesCfg );

		InputManager		*mInputMgr;
		Ogre::Root			*mRoot;
		Ogre::RenderWindow	*mRenderWindow;
		GameStateManager	*mStateMgr;
		EventManager		*mEventMgr;

		bool bShutdown;
	};
}

