#include "GameStateManager.h"
#include "IEventManager.h"


namespace GardeGames {

	GameStateManager::GameStateManager(InputManager *inputManager)
    {
        this->inputManager = inputManager;
		currentState=0;
		done=false;
    }	

	GameStateManager::~GameStateManager(void)
	{
	}

    void GameStateManager::addState(GameState* state) {
		gameStates[state->getName()]=state;
    }

	/*std::iterator GameStateManager::getStates() {
        gameStates
    }*/

    /*public void loadAllResources(ResourceManager resourceManager) {
        Iterator i = getStates();
        while (i.hasNext()) {
            GameState gameState = (GameState)i.next();
            gameState.loadResources(resourceManager);
        }
    }*/


    bool GameStateManager::isDone() {
        return done;
    }


    /**
        Sets the current state (by name).
    */
    void GameStateManager::setState(std::string name) {
        // clean up old state
		if (currentState) {
            currentState->stop();
        }
        inputManager->clearAllMaps();

		if (name.compare(exitGame())==0) {
            done = true;
        }
        else {

            // set new state
			currentState = gameStates[name];
            if (currentState != 0) {
                currentState->start(inputManager);
            }
        }
    }


    /**
        Updates world, handles input.
    */
    void GameStateManager::update(long elapsedTime) {
		std::string nextState = currentState->checkForStateChange();
		if (!nextState.empty()) {
            setState(nextState);
        }
        else {
            currentState->update(elapsedTime);
        }
    }

	std::string GameStateManager::exitGame() {
		return "_EXIT_GAME_";
	}
}