/*
 * This file is part of ElSoldadoYLaElfa.
 *
 * 
 * Miguel Garde Vallés (C) 2012
 * <miguelgardev@gmail.com>
 *
 * 
 * ElSoldadoYLaElfa examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * ElSoldadoYLaElfa examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ElSoldadoYLaElfa examples.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "GardeGames/GardeGames.h"
#include "Ogre.h"
#include "Creature.h"
#include <Ogre/SdkTrays.h>

//! Clase que gestiona el comportamiento del jugador humano.


/**
 *  @author Miguel Garde Vallés
 *  @date 14-01-2012
 *
 *  Implementa IGameView para controlar todo lo relacionado con el personaje humano:
 *  cuándo puede atacar, cuándo debe parar de andar, cuándo reproducir la animación de muerte, etc.
 *  Así mismo gestiona la cámara, tanto el seguimiento del personaje como los límites de giro.
 *  También se encarga de disparar eventos, tanto de movimiento como de ataques y sonido
 */

class HumanView : public GardeGames::IGameView
{
	friend class HumanViewListener;
public:

	enum HumanStates {
		IDLE,
		FACING_ATTACK1,
		FACING_ATTACK2,
		ATTACKING1,
                ATTACKING2
	};

	HumanView(Creature *elfa,
                                Ogre::PanelOverlayElement *lifeBar,
				GardeGames::GameAction *accionAlante,
				GardeGames::GameAction *accionAtras,
				GardeGames::GameAction *accionIzda,
				GardeGames::GameAction *accionDcha,				
				GardeGames::GameAction *accionRatonMovido,
				GardeGames::GameAction *accionGolpea,
				GardeGames::GameAction *accionLanzaRayo,
				GardeGames::GameAction *accionElfa,
				GardeGames::GameAction *accionLobo,
				GardeGames::GameAction *accionCiervo,
				Ogre::Camera* cam
				);
	~HumanView(void);

	//heredadas
	GameViewType VGetType(){return IGameView::GameView_Human;};
	GameViewId VGetId() const {return 0;};
	void VOnUpdate( int deltaMilliseconds );
        
        void adjustCamHeight(Ogre::Real newHeight);
        Ogre::Vector3 getCamPosition();        
        Ogre::Vector3 getPivotPosition();
	void updateCameraGoal(Ogre::Real deltaYaw, Ogre::Real deltaPitch, Ogre::Real deltaZoom);
        
        Creature* getPlayer();
        
        Ogre::PanelOverlayElement *mLifeBar;
	
private:
	Creature *mJugador;
        Creature *mElfa;
        Creature *mLobo;
        Creature *mCiervo;

	HumanStates mEstadoActual;

	//acciones de movimiento
	GardeGames::GameAction *mAccionAlante;
	GardeGames::GameAction *mAccionAtras;
	GardeGames::GameAction *mAccionIzda;
	GardeGames::GameAction *mAccionDcha;
	GardeGames::GameAction *mAccionRatonMovido;

	//acciones de ataque
	GardeGames::GameAction *mAccionGolpea;
	GardeGames::GameAction *mAccionLanzaRayo;
        
        //acciones de transformacion
	GardeGames::GameAction *mAccionTransElfa;
	GardeGames::GameAction *mAccionTransCiervo;
	GardeGames::GameAction *mAccionTransLobo;

	Ogre::SceneNode* mCameraPivot;
	Ogre::SceneNode* mCameraGoal;
	Ogre::SceneNode* mCameraNode;
	float mPivotPitch;
        bool mIsCameraAdjusting; 
        float mNewHeight;

	Ogre::SceneNode* mRayoPivot;
	Ogre::ParticleSystem* mRayo;
	float mTiempoRayo;

	void updateCamera(Ogre::Real deltaTime);
        void cambiarForma(Creature* nuevaForma);
	Ogre::Real atacar(Ogre::Real deltaTime);
};
