/*
 * This file is part of ElSoldadoYLaElfa.
 *
 * 
 * Héctor Agustín Lana (C) 2012
 * <hectoragustin89@hotmail.com
 *
 * 
 * ElSoldadoYLaElfa examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * ElSoldadoYLaElfa examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ElSoldadoYLaElfa examples.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "SteeringBehaviors.h"
#include "utils.h"
#include <fstream>
#include <iostream>
#include <string>
#include <algorithm>

using namespace std; 



SteeringBehaviors::SteeringBehaviors(AIView*  aiView)
                                             
{
   //INICIALIZAR FLAGS

   bArrive=false;
   bSeek=false;
   bFlee=false;
   bWander=false;
   bObstacleAvoidance=false;
   bFollowPath=false;



   obstacleAvoidanceWeight=0.3;
   priorityThreshold=50;

   //radio para el  circulo en el comportamiento wander
   WanderRad    = 200;
   //distancia a la que es proyectada el circulo en frente del agente
   WanderDist   = 100;
   //la maxima cantidad de desplazamiento en cada  frame
   WanderJitterPerSec = 1200.0;
   mAiView=aiView;	
    //angulo para wander

   double theta = RandFloat() * TwoPi;


   //posicion inicial del target en el comportamiento wander
   m_vWanderTarget = Ogre::Vector2(WanderRad * Ogre::Math::Cos(theta),WanderRad * Ogre::Math::Sin(theta));

}

SteeringBehaviors::~SteeringBehaviors()
  {

  }
  void SteeringBehaviors::seekOn()
     {
	    bSeek=true;
      }
  void SteeringBehaviors::seekOff()
     {
	   bSeek=false;
     }
  bool SteeringBehaviors::isSeek()
     {
	   return bSeek;
     }
    void SteeringBehaviors::arriveOn()
     {
	    bArrive=true;
      }
  void SteeringBehaviors::arriveOff()
     {
	   bArrive=false;
     }
  bool SteeringBehaviors::isArrive()
     {
	   return bArrive;
     }
  void SteeringBehaviors::fleeOn()
     {
		 bFlee=true;
      }
  void SteeringBehaviors::fleeOff()
    {
		 bFlee=false;
    }
  bool SteeringBehaviors::isFlee()
     {
	   return bFlee;
      }
  void SteeringBehaviors::wanderOn()
     {
	    bWander=true;
      }
  void SteeringBehaviors::wanderOff()
     {
	   bWander=false;
     }
  bool SteeringBehaviors::isWander() 
      {
	     return bWander;
      }
  void SteeringBehaviors::obstacleAvoidanceOn()
      {
	    bObstacleAvoidance=true;
      }
  void SteeringBehaviors::obstacleAvoidanceOff()
       {
	     bObstacleAvoidance=false;
       }
  bool SteeringBehaviors::isObstacleAvoidance()
       {
		   return bObstacleAvoidance;
       }
  void SteeringBehaviors::followPathOn()
    {
         bFollowPath=true;
    }
  void SteeringBehaviors::followPathOff()
  {
      bFollowPath=false;
  }
  bool SteeringBehaviors::isfollowPath()
  {
       return bFollowPath;
  }


//------------------------------- Seek -----------------------------------
  //para perseguir a un enemigo
//------------------------------------------------------------------------



  Ogre::Vector2 SteeringBehaviors::Seek(Ogre::Vector2 target)
  {
	  Ogre::Vector2 tarr=target;
	  Ogre::Vector2 velo=mAiView->getVelocity();
		//calculamos la velocidad deseada
	  double posx=mAiView->getCreature()->getPosition().x;
	  double posz=mAiView->getCreature()->getPosition().z;
	  Ogre::Vector2 dv=Ogre::Vector2(-(target.x-mAiView->getCreature()->getPosition().x),target.y-mAiView->getCreature()->getPosition().z);
		dv.normalise(); 

		Ogre::Vector2 desiredVelocity =dv*mAiView->getCreature()->getMaxSpeed();

	    mAiView->desiredVelocityX=desiredVelocity.x;
		mAiView->desiredVelocityY=desiredVelocity.y;
	
		Ogre::Vector2 aux=Ogre::Vector2(desiredVelocity.x-mAiView->getVelocity().x,desiredVelocity.y-mAiView->getVelocity().y);
		return aux;
  }

  Ogre::Vector2 SteeringBehaviors::Evade(Ogre::Vector2 target)
  {
	  
		//calculamos la velocidad deseada
	  Ogre::Vector2 dv=Ogre::Vector2(-(mAiView->getCreature()->getPosition().x-target.x),mAiView->getCreature()->getPosition().z-target.y);
		dv.normalise(); 
		Ogre::Vector2 desiredVelocity =dv*mAiView->getCreature()->getMaxSpeed();
		return Ogre::Vector2(desiredVelocity.x-mAiView->getVelocity().x,desiredVelocity.y-mAiView->getVelocity().y);
		
  }



//--------------------------- Wander -------------------------------------
//
//  Para vagar aleatoriamente
//------------------------------------------------------------------------
Ogre::Vector2  SteeringBehaviors::Wander()
{ 
 //calibrar el tiempo entre frames
  double JitterThisTimeSlice =  WanderJitterPerSec * mAiView->getTimeSinceLastFrame();

  //anadir un vector aleatorio a la posicion del target
  m_vWanderTarget +=Ogre::Vector2(RandomClamped() * JitterThisTimeSlice,RandomClamped() * JitterThisTimeSlice);

  //reproyectar el vector en torno a un circulo unitario
  m_vWanderTarget.normalise();

//incrementar la longitud del vector como la del circulo
  m_vWanderTarget *= WanderRad;

  //mover al target una distancia WanderDist en frente del circulo
  Ogre::Vector2 target = m_vWanderTarget + Ogre::Vector2(0,WanderDist);

  //proyectar el target en el world space
  Ogre::Vector2 Target = globalPosition(target,mAiView->getCreature()->getOrientation(),mAiView->getCreature()->getPosition());
 
  //girar

  Ogre::Vector2 aux=Ogre::Vector2(-(Target.x-mAiView->getCreature()->getPosition().x),Target.y-mAiView->getCreature()->getPosition().z);

  return aux;
}

//comportamiento para evitar obstaculos
Ogre::Vector2  SteeringBehaviors::ObstacleAvoidance(Ogre::Vector2 obstaclePosition,double obstacleRadius)
{
	double multiplier=1;
	Ogre::Vector2 steeringForce;
	//calculate  lateral force
	Ogre::Vector2	localObstaclePosition=localPosition(obstaclePosition,mAiView->getCreature()->getPosition(),mAiView->getCreature()->getOrientation());
	steeringForce.x=(obstacleRadius-localObstaclePosition.x)*multiplier;

	//calculate the braking force
	double brakingWeight=0.2;
	
	steeringForce.y=(obstacleRadius-localObstaclePosition.y)*brakingWeight;

	return  globalPosition(steeringForce,mAiView->getCreature()->getOrientation(),mAiView->getCreature()->getPosition());

}
//comportamiento para calcular la steering force
Ogre::Vector2  SteeringBehaviors::CalculateSteeringForce()
{

	 Ogre::Vector2 steeringForce;

	 if(isWander())
	 {
		steeringForce=Wander();
	 }
         else if (isArrive())
         {
             steeringForce=Seek(Ogre::Vector2(mAiView->getArrivingPoint().x,mAiView->getArrivingPoint().z));
             
         }
	 else if (isSeek())

	 {
		 steeringForce=Seek(Ogre::Vector2(mAiView->getTarget()->getPosition().x,mAiView->getTarget()->getPosition().z));
	 }

         else if  (isFlee())
	 {
		 steeringForce=Evade(Ogre::Vector2(mAiView->getTarget()->getPosition().x,mAiView->getTarget()->getPosition().z));
	 }
        else //evade
	 {
		 steeringForce=FollowPath(&mAiView->_path);  
	 }
   return steeringForce;
}
//algoritmo de prioridades para combinar steering behaviors
Ogre::Vector2 SteeringBehaviors::PrioritiesAlgorithm()
{
	Ogre::Vector2 steeringForce=CalculateSteeringForce();
	if (mAiView->isClosestObstacle())
	{
		Ogre::Vector2 obstacleAvoidanceForce=ObstacleAvoidance(Ogre::Vector2(mAiView->closestObstacle->getPosition().x,mAiView->closestObstacle->getPosition().z),mAiView->closestObstacle->getRadius());
		if (obstacleAvoidanceForce.length()>priorityThreshold)
		{
			return obstacleAvoidanceForce;
		}
		else
		{
			return steeringForce;
		}
	}
	else
	{
      return steeringForce;
	}
}


//algoritmo de medias para combinar steering behaviors
Ogre::Vector2 SteeringBehaviors::WeightedSumAlgorithm()
{
	Ogre::Vector2 steeringForce=CalculateSteeringForce();
	if (mAiView->closestObstacle)
	{
      
	   Ogre::Vector2 obstacleAvoidanceForce=ObstacleAvoidance(Ogre::Vector2(mAiView->closestObstacle->getPosition().x,mAiView->closestObstacle->getPosition().z),mAiView->closestObstacle->getRadius());
	   return Ogre::Vector2((1- obstacleAvoidanceWeight)*steeringForce.x+obstacleAvoidanceForce.x*obstacleAvoidanceWeight,(1- obstacleAvoidanceWeight)*steeringForce.y+obstacleAvoidanceForce.y*obstacleAvoidanceWeight);
	}
	else
	     { 
			 return steeringForce;
	     }

}

//encuentra el destino en un camino
Ogre::Vector3 SteeringBehaviors::findTargetInPath(navigationMesh::PointPath* mPath) {
    Ogre::Real minDistance = Ogre::Math::POS_INFINITY;
    navigationMesh::PointPath::iterator closestPointIt = mPath->begin(); 
    
    // Recorremos la lista de puntos buscando el más cercano
    for (navigationMesh::PointPath::iterator i = mPath->begin(); i != mPath->end(); ++i) {
        // Calculamos la nueva distancia
        Ogre::Vector3 direction = mAiView->getCreature()->getPosition() - *i;
        
        if (direction.squaredLength() < minDistance) {
            minDistance = direction.squaredLength();
            closestPointIt = i;
        }
    }
    
    navigationMesh::PointPath::iterator targetIt = closestPointIt;
    ++targetIt;
    
    if (targetIt == mPath->end())
        return *closestPointIt;
    
    return *targetIt;
}


//comportamiento que sigue un path
 Ogre::Vector2 SteeringBehaviors::FollowPath(navigationMesh::PointPath* path)
 {
     Ogre::Vector3 aux=findTargetInPath(path);
     return Seek(Ogre::Vector2(aux.x,aux.z));
 }


