/*
 * This file is part of SionTower.
 *
 * 
 * David Saltares M&aacute;rquez (C) 2011
 * <david.saltares@gmail.com>
 *
 * 
 * SionTower examples are free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License ad
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) ant later version.
 *
 * SionTower examples are distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SionTower examples.  If not, see <http://www.gnu.org/licenses/>.
 */




#pragma once

#include <vector>
#include <list>

#include <OGRE/Ogre.h>

#include "cell.h"

class navigationMesh {
    public:
        /** Conjunto de celdas que forman el grafo*/
        typedef std::vector<Cell*> Cells;
        
        /** Camino formado por una lista de puntos */
        typedef std::list<Ogre::Vector3> PointPath; 
        
        /** Camino formado por una lista de celdas */
        typedef std::list<Cell*> CellPath;
        
      
        

        navigationMesh(Ogre::SceneManager *mSceneMgr,const Ogre::String& fileName = "");
        
        /**
         * Destructor
         */
        ~navigationMesh();
        
   
        void addCell(int id,
                     const Ogre::Vector3& pointA,
                     const Ogre::Vector3& pointB,
                     const Ogre::Vector3& pointC);
        
     
        void linkCells();
        
        
        //Elimina  las celdas de la malla.
         
        void clear();
        
        //obtiene un puntero a la celda correspondiente
        Cell* getCell(int index);
        
        //devuelve el numero de celdas que tiene la malla
        int getCellNumber();

        /**
         * Reconstruye el camino a partir de las rutas producidas por el
         * algoritmo de Floyd y aplica un spline
         * creando puntos intermedios para suavizar la ruta.
         * 
         *path camino de puntos (salida)
         *startPos-posición de comienzo (debe estar en la malla)
         *endPos posición de destino (debe estar en la malla)
         *startCell celda de inicio para mayor eficiencia. Si no
         *se facilita se buscará automáticamente (más lento).
         *endCell celda final para mayor eficiencia. Si no
         *se facilita se buscará automáticamente (más lento).
         * 
         * @return true si existe camino, false en caso contrario
         */
        bool buildPath(PointPath& path,
                       const Ogre::Vector3& startPos,
                       const Ogre::Vector3& endPos,
                       Cell* startCell = 0,
                       Cell* endCell = 0);
        
        //devuelve la celda mas cercana a la posicion
        Cell* findCell(const Ogre::Vector3& pos);
        
        /**
         *start punto de comienzo
         *end punto final
         *startCell celda de comienzo
         *endCell celda final
         * 
         * @return true si hay línea de visión entre los dos puntos, false
         * en caso contrario.
         */
        bool lineOfSightTest(const Ogre::Vector3& start,
                             const Ogre::Vector3& end,
                             Cell* startCell,
                             Cell* endCell);
        
        /**
         *path camino
         *startIt iterador dentro del camino dado
         * 
         *iterador del camino apuntando a la celda más lejana
         * con la que haya una línea de visión directa.
         */
        CellPath::iterator getFurthestVisibleCell(CellPath& path, CellPath::iterator startIt);
        
    private:
        void loadCellsFromXML(const Ogre::String& fileName);
        bool makeSpline(PointPath& path);
        int simplifyPath(CellPath& cellPath);
        Ogre::SceneManager *_SceneMgr;
        Cells _cells;
        int _cellNumber;
        
        // Grafo y Floyd
        Ogre::Real* _graph;
        int* _paths;
        
        void initGraph();
        void floyd();
        //mejoramos los caminos
        void improvePaths();
        void recoverPath(int i, int j, CellPath& cellPath);
};

